"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime/helpers/possibleConstructorReturn"));

var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));

var _get2 = _interopRequireDefault(require("@babel/runtime/helpers/get"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _assertThisInitialized2 = _interopRequireDefault(require("@babel/runtime/helpers/assertThisInitialized"));

var THREE = _interopRequireWildcard(require("three"));

var _FirstPersonControls2 = _interopRequireDefault(require("./FirstPersonControls"));

var _AnimationPlayer = _interopRequireWildcard(require("../Core/AnimationPlayer"));

// Expression used to damp camera's moves
function moveCameraExp(root, progress) {
  var dampingProgress = 1 - Math.pow(1 - (Math.sin((progress - 0.5) * Math.PI) * 0.5 + 0.5), 2);
  root.camera.position.lerpVectors(root.positionFrom, root.positionTo, dampingProgress);
}
/**
 * @classdesc Camera controls that can follow a path.
 * It is used to simulate a street view.
 * It stores a currentPosition and nextPosition, and do a camera traveling to go to next position.
 *
 * <ul> Bindings inherited from FirstPersonControls
 * <li><b> up + down keys : </b> forward/backward </li>
 * <li><b> left + right keys: </b> strafing movements </li>
 * <li><b> pageUp + pageDown: </b> vertical movements </li>
 * <li><b> mouse click+drag: </b> pitch and yaw movements (as looking at a panorama) </li>
 * </ul>
 * <ul> Bindings added
 * <li><b> keys Z : </b> Move camera to the next position </li>
 * <li><b> keys S : </b> Move camera to the previous position </li>
 * <li><b> keys A : </b> Set camera to current position </li>
 * </ul>
 * @property {number} keyGoToNextPosition key code to go to next position, default to 90 for key Z
 * @property {number} keyGoToPreviousPosition key code to go to previous position, default to 83 for key S
 * @property {number} keySetCameraToCurrentPosition key code set camera to current position, default to 65 for key  A
 * @extends FirstPersonControls
 */


var StreetControls =
/*#__PURE__*/
function (_FirstPersonControls) {
  (0, _inherits2.default)(StreetControls, _FirstPersonControls);

  /**
   * @constructor
   * @param { View } view - View where this control will be used
   * @param { Object } options - Configuration of this controls
   * @param { number } options.animationDuration - Duration of the animation to another panoramic.
   */
  function StreetControls(view) {
    var _this;

    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    (0, _classCallCheck2.default)(this, StreetControls);
    _this = (0, _possibleConstructorReturn2.default)(this, (0, _getPrototypeOf2.default)(StreetControls).call(this, view, options));
    _this.isStreetControls = true; // store layers

    _this.layers = [];
    _this.currentLayerIndex = 0; // manage camera movements

    _this.player = new _AnimationPlayer.default();
    _this.positionFrom = new THREE.Vector3();
    _this.animationMoveCamera = new _AnimationPlayer.AnimatedExpression({
      duration: options.animationDuration || 50,
      root: (0, _assertThisInitialized2.default)((0, _assertThisInitialized2.default)(_this)),
      expression: moveCameraExp,
      name: 'Move camera'
    });

    _this.player.addEventListener('animation-frame', _this.updateView.bind((0, _assertThisInitialized2.default)((0, _assertThisInitialized2.default)(_this)))); // twos positions used by this control : current and next


    _this.previousPosition = undefined;
    _this.currentPosition = undefined;
    _this.nextPosition = undefined;
    _this.keyGoToNextPosition = 90;
    _this.keyGoToPreviousPosition = 83;
    _this.keySetCameraToCurrentPosition = 65;
    return _this;
  }

  (0, _createClass2.default)(StreetControls, [{
    key: "setCurrentPosition",
    value: function setCurrentPosition(newCurrentPosition) {
      this.currentPosition = newCurrentPosition;
    }
  }, {
    key: "setNextPosition",
    value: function setNextPosition(newNextPosition) {
      this.nextPosition = newNextPosition;
    }
  }, {
    key: "setPreviousPosition",
    value: function setPreviousPosition(newPreviousPosition) {
      this.previousPosition = newPreviousPosition;
    }
    /**
     * Sets the camera to the current position, looking at the next position.
     */

  }, {
    key: "setCameraToCurrentPosition",
    value: function setCameraToCurrentPosition() {
      this.setCameraOnPosition(this.currentPosition, this.nextPosition);
    }
    /**
     * Set the camera on a position, looking at another position.
     *
     * @param      { THREE.Vector3 }  position   The position to set the camera
     * @param      { THREE.Vector3 }  lookAt      The position where the camera look at.
     */

  }, {
    key: "setCameraOnPosition",
    value: function setCameraOnPosition(position, lookAt) {
      if (!position || !lookAt) {
        return;
      }

      this.camera.position.copy(position);
      this.camera.up.copy(position).normalize();
      this.camera.lookAt(lookAt);
      this.camera.updateMatrixWorld();
      this.reset();
    }
    /**
     * Move the camera
     *
     * @param { THREE.Vector3 }  positionTo  Destination of the movement.
     */

  }, {
    key: "moveCameraTo",
    value: function moveCameraTo(positionTo) {
      if (!positionTo) {
        return;
      }

      this.positionFrom.copy(this.camera.position);
      this.positionTo = positionTo;
      this.player.play(this.animationMoveCamera);
    }
  }, {
    key: "moveCameraToCurrentPosition",
    value: function moveCameraToCurrentPosition() {
      this.moveCameraTo(this.currentPosition);
    }
  }, {
    key: "onKeyDown",
    value: function onKeyDown(e) {
      (0, _get2.default)((0, _getPrototypeOf2.default)(StreetControls.prototype), "onKeyDown", this).call(this, e); // key to move to next position (default to Z)

      if (e.keyCode == this.keyGoToNextPosition) {
        this.moveCameraTo(this.nextPosition);
      } // key to move to previous position (default to S)


      if (e.keyCode == this.keyGoToPreviousPosition) {
        this.moveCameraTo(this.previousPosition);
      } // key to set to camera to current position looking at next position (default to A)


      if (e.keyCode == this.keySetCameraToCurrentPosition) {
        this.setCameraToCurrentPosition();
        this.view.notifyChange(this.view.camera.camera3D);
      }
    }
  }, {
    key: "updateView",
    value: function updateView() {
      this.view.notifyChange(this.camera);
    }
  }]);
  return StreetControls;
}(_FirstPersonControls2.default);

var _default = StreetControls;
exports.default = _default;